#pragma once

#ifdef __cplusplus

class ATL_NO_VTABLE CReferencedDocument : 
	public CComObjectRootEx<CComMultiThreadModel>,
	public CDocumentDataImpl<CReferencedDocument, &GUID_NULL, EUMAllSteps>,
	public IDocumentSoftLink
{
public:
	void Init(IStorageFilter* a_pLocation, IDocument* a_pDocument)
	{
		m_pLocation = a_pLocation;
		m_pDocument = a_pDocument;
	}

	static HRESULT WINAPI QICustomInterface(void* pv, REFIID riid, LPVOID* ppv, DWORD_PTR dw)
	{
		CReferencedDocument* const p = reinterpret_cast<CReferencedDocument*>(pv);
		if (p->m_pDocument)
			return p->m_pDocument->QueryFeatureInterface(riid, ppv);
		return E_NOINTERFACE;
	}

BEGIN_COM_MAP(CReferencedDocument)
	COM_INTERFACE_ENTRY(IDocumentData)
	COM_INTERFACE_ENTRY(IDocumentSoftLink)
	COM_INTERFACE_ENTRY_FUNC_BLIND(0, QICustomInterface)
END_COM_MAP()

	// IDocumentData methods
public:
	STDMETHOD(Aspects)(IEnumEncoderAspects* a_pEnumAspects)
	{
		return S_OK;
	}

	STDMETHOD(WriteFinished)()
	{
		return S_OK;
	}

	STDMETHOD(DataCopy)(EDocumentCopyHint a_eHint, BSTR a_bstrPrefix, IDocumentBase* a_pBase)
	{
		try
		{
			CComObject<CReferencedDocument>* p = NULL;
			CComObject<CReferencedDocument>::CreateInstance(&p);
			CComPtr<IDocumentData> pData = p;
			p->Init(m_pLocation, m_pDocument);
			return a_pBase->DataBlockSet(a_bstrPrefix, pData);
		}
		catch (...)
		{
			return E_UNEXPECTED;
		}
	}

	//STDMETHOD(LocationChanged)(IStorageFilter* a_pOldLoc);
	//STDMETHOD(RemovingBlock)();

	//STDMETHOD(ResourcesManage)(EDocumentResourceManager a_eActions, ULONGLONG* a_pValue);

	// IDocumentSoftLink methods
public:
	STDMETHOD(Location)(IStorageFilter** a_ppLoc)
	{
		(*a_ppLoc = m_pLocation)->AddRef();
		return S_OK;
	}

private:
	CComPtr<IStorageFilter> m_pLocation;
	CComPtr<IDocument> m_pDocument;
};

class CReferencedDocumentCreator :
	public ISceneObjectCreator
{
public:
	CReferencedDocumentCreator(IStorageFilter* a_pRoot, BSTR a_bstrFilePath) :
		m_pRoot(a_pRoot), m_bstrFilePath(a_bstrFilePath)
	{
	}

	// IUnknown methods
public:
	STDMETHOD(QueryInterface)(REFIID a_riid, void** a_ppvObject)
	{
		if (IsEqualIID(a_riid, IID_IUnknown) || IsEqualIID(a_riid, __uuidof(ISceneObjectCreator)))
		{
			*a_ppvObject = this;
			return S_OK;
		}
		return E_NOINTERFACE;
	}
	STDMETHOD_(ULONG, AddRef)() { return 2; }
	STDMETHOD_(ULONG, Release)() { return 1; }

	// ISceneObjectCreator methods
public:
	STDMETHOD(Create)(IDocumentBuilder* a_pDefaultBuilder, BSTR a_bstrID, IDocumentBase* a_pBase)
	{
		CComPtr<IStorageManager> pSM;
		RWCoCreateInstance(pSM, __uuidof(StorageManager));
		CComPtr<IStorageFilter> pLoc;
		pSM->FilterCreateEx(m_pRoot, m_bstrFilePath, 0, &pLoc);
		CComPtr<IInputManager> pIM;
		RWCoCreateInstance(pIM, __uuidof(InputManager));
		CComPtr<IEnumUnknowns> pBuilders;
		pIM->GetCompatibleBuilders(1, &__uuidof(ISceneSegment), &pBuilders);
		CComPtr<IDocument> pDoc;
		RWCoCreateInstance(pDoc, __uuidof(DocumentBase));
		pIM->DocumentCreateData(pBuilders, pLoc, NULL, CComQIPtr<IDocumentBase>(pDoc));
		CComObject<CReferencedDocument>* p = NULL;
		CComObject<CReferencedDocument>::CreateInstance(&p);
		CComPtr<IDocumentData> pData = p;
		p->Init(pLoc, pDoc);
		return a_pBase->DataBlockSet(a_bstrID, pData);
	}

private:
	CComPtr<IStorageFilter> m_pRoot;
	CComBSTR m_bstrFilePath;
};

class CSceneObjectCreatorStock :
	public ISceneObjectCreator
{
public:
	CSceneObjectCreatorStock(ULONG a_nType) : m_nType(a_nType)
	{
	}

	// IUnknown methods
public:
	STDMETHOD(QueryInterface)(REFIID a_riid, void** a_ppvObject)
	{
		if (IsEqualIID(a_riid, IID_IUnknown) || IsEqualIID(a_riid, __uuidof(ISceneObjectCreator)))
		{
			*a_ppvObject = this;
			return S_OK;
		}
		return E_NOINTERFACE;
	}
	STDMETHOD_(ULONG, AddRef)() { return 2; }
	STDMETHOD_(ULONG, Release)() { return 1; }

	// ISceneObjectCreator methods
public:
	STDMETHOD(Create)(IDocumentBuilder* a_pDefaultBuilder, BSTR a_bstrID, IDocumentBase* a_pBase)
	{
		CComPtr<IDocumentFactoryStock3DObject> pFact;
		RWCoCreateInstance(pFact, __uuidof(DocumentFactoryStock3DObject));
		return pFact ? pFact->Create(m_nType, a_bstrID, a_pBase) : E_FAIL;
	}

private:
	ULONG m_nType;
};


#endif//__cplusplus

