
#pragma once


inline HRESULT RGBAGetTileImpl(TImageSize const& a_tSrcCanvasSize, TImagePoint const& a_tSrcOrigin, TImageSize const& a_tSrcSize, TPixelChannel const* a_pSrc, ULONG a_nSrcStrideY, TPixelChannel const a_tDefault, TImagePoint const* a_pOrigin, TImageSize const* a_pSize, TImageStride const* a_pStride, ULONG a_nPixels, TPixelChannel* a_pPixels)
{
	if (a_pOrigin && a_pSize == NULL)
		return E_RW_INVALIDPARAM; // if Origin is set, so must be Size
	TImagePoint tOrigin = {0, 0};
	if (a_pOrigin) tOrigin = *a_pOrigin;
	TImagePoint const tEnd =
	{
		a_pSize ? tOrigin.nX+a_pSize->nX : tOrigin.nX+a_tSrcCanvasSize.nX,
		a_pSize ? tOrigin.nY+a_pSize->nY : tOrigin.nY+a_tSrcCanvasSize.nY
	};
	TImageSize const tSize = {tEnd.nX-tOrigin.nX, tEnd.nY-tOrigin.nY};

	TImageSize const tStride = {a_pStride ? a_pStride->nX : 1, a_pStride ? a_pStride->nY : tEnd.nX-tOrigin.nX};
	ULONG const nLineDiff = tStride.nY-tStride.nX*tSize.nX;
	if (a_nPixels < tStride.nY*tSize.nY)
		return E_FAIL; // buffer not big enough

	TImagePoint const tSrcEnd = {a_tSrcOrigin.nX+a_tSrcSize.nX, a_tSrcOrigin.nY+a_tSrcSize.nY};

	if (tOrigin.nX >= tSrcEnd.nX || tOrigin.nY >= tSrcEnd.nY || tEnd.nX <= a_tSrcOrigin.nX || tEnd.nY <= a_tSrcOrigin.nY)
	{
		// empty tile
		for (; tOrigin.nY < tEnd.nY; ++tOrigin.nY)
		{
			for (TPixelChannel* const pEnd = a_pPixels+tStride.nX*tSize.nX; a_pPixels != pEnd; a_pPixels += tStride.nX)
				*a_pPixels = a_tDefault;
			a_pPixels += nLineDiff;
		}
		return S_OK;
	}

	for (LONG const nYEnd = tEnd.nY < a_tSrcOrigin.nY ? tEnd.nY : a_tSrcOrigin.nY; tOrigin.nY < nYEnd; ++tOrigin.nY)
	{
		for (TPixelChannel* const pEnd = a_pPixels+tStride.nX*tSize.nX; a_pPixels != pEnd; a_pPixels += tStride.nX)
			*a_pPixels = a_tDefault;
		a_pPixels += nLineDiff;
	}
	LONG const nXLen1 = tEnd.nX < a_tSrcOrigin.nX ? tEnd.nX-tOrigin.nX : (tOrigin.nX < a_tSrcOrigin.nX ? a_tSrcOrigin.nX-tOrigin.nX : 0);
	LONG const nXLen3 = tEnd.nX > tSrcEnd.nX ? tEnd.nX-tSrcEnd.nX : 0;
	LONG const nXLen2 = tSize.nX-nXLen1-nXLen3;
	TPixelChannel const* pSrc = a_pSrc+(tOrigin.nY-a_tSrcOrigin.nY)*a_nSrcStrideY+(tOrigin.nX > a_tSrcOrigin.nX ? tOrigin.nX-a_tSrcOrigin.nX : 0);
	ULONG const tSrcLineDiff = a_nSrcStrideY-nXLen2;
	for (LONG const nYEnd = tEnd.nY < tSrcEnd.nY ? tEnd.nY : tSrcEnd.nY; tOrigin.nY < nYEnd; ++tOrigin.nY)
	{
		for (TPixelChannel* const pEnd = a_pPixels+tStride.nX*nXLen1; a_pPixels != pEnd; a_pPixels += tStride.nX)
			*a_pPixels = a_tDefault;
		for (TPixelChannel* const pEnd = a_pPixels+tStride.nX*nXLen2; a_pPixels != pEnd; a_pPixels += tStride.nX, ++pSrc)
			*a_pPixels = *pSrc;
		for (TPixelChannel* const pEnd = a_pPixels+tStride.nX*nXLen3; a_pPixels != pEnd; a_pPixels += tStride.nX)
			*a_pPixels = a_tDefault;
		a_pPixels += nLineDiff;
		pSrc += tSrcLineDiff;
	}
	for (; tOrigin.nY < tEnd.nY; ++tOrigin.nY)
	{
		for (TPixelChannel* const pEnd = a_pPixels+tStride.nX*tSize.nX; a_pPixels != pEnd; a_pPixels += tStride.nX)
			*a_pPixels = a_tDefault;
		a_pPixels += nLineDiff;
	}
	return S_OK;
}

inline HRESULT RGBAInspectImpl(TImagePoint const& a_tSrcOrigin, TImageSize const& a_tSrcSize, TPixelChannel const* a_pSrc, ULONG a_nSrcStrideY, TPixelChannel const a_tDefault, TImagePoint const* a_pOrigin, TImageSize const* a_pSize, IImageVisitor* a_pVisitor, ITaskControl* a_pControl)
{
	if (a_pOrigin == NULL || a_pSize == NULL)
	{
		a_pOrigin = &a_tSrcOrigin;
		a_pSize = &a_tSrcSize;
	}
	TImageTile aTiles[5];
	ULONG nTiles = 0;
	TImagePoint const tSrc1 = {a_pOrigin->nX < a_tSrcOrigin.nX ? a_tSrcOrigin.nX : a_pOrigin->nX, a_pOrigin->nY < a_tSrcOrigin.nY ? a_tSrcOrigin.nY : a_pOrigin->nY};
	TImagePoint const tSrc2 = {LONG(a_pOrigin->nX+a_pSize->nX) > LONG(a_tSrcOrigin.nX+a_tSrcSize.nX) ? a_tSrcOrigin.nX+a_tSrcSize.nX : a_pOrigin->nX+a_pSize->nX, LONG(a_pOrigin->nY+a_pSize->nY) > LONG(a_tSrcOrigin.nY+a_tSrcSize.nY) ? a_tSrcOrigin.nY+a_tSrcSize.nY : a_pOrigin->nY+a_pSize->nY};
	if (tSrc1.nX < tSrc2.nX && tSrc1.nY < tSrc2.nY)
	{
		if (a_pOrigin->nY < tSrc1.nY)
		{
			// top
			aTiles[nTiles].nChannelIDs = EICIRGBA;
			aTiles[nTiles].tOrigin = *a_pOrigin;
			aTiles[nTiles].tSize.nX = a_pSize->nX;
			aTiles[nTiles].tSize.nY = tSrc1.nY-a_pOrigin->nY;
			aTiles[nTiles].tStride.nX = aTiles[nTiles].tStride.nY = 0;
			aTiles[nTiles].nPixels = 1;
			aTiles[nTiles].pData = &a_tDefault;
			++nTiles;
		}
		if (a_pOrigin->nX < tSrc1.nX)
		{
			// left
			aTiles[nTiles].nChannelIDs = EICIRGBA;
			aTiles[nTiles].tOrigin.nX = a_pOrigin->nX;
			aTiles[nTiles].tOrigin.nY = tSrc1.nY;
			aTiles[nTiles].tSize.nX = tSrc1.nX-a_pOrigin->nX;
			aTiles[nTiles].tSize.nY = tSrc2.nY-tSrc1.nY;
			aTiles[nTiles].tStride.nX = aTiles[nTiles].tStride.nY = 0;
			aTiles[nTiles].nPixels = 1;
			aTiles[nTiles].pData = &a_tDefault;
			++nTiles;
		}
		{
			// center = content
			aTiles[nTiles].nChannelIDs = EICIRGBA;
			aTiles[nTiles].tOrigin = tSrc1;
			aTiles[nTiles].tSize.nX = tSrc2.nX-tSrc1.nX;
			aTiles[nTiles].tSize.nY = tSrc2.nY-tSrc1.nY;
			aTiles[nTiles].tStride.nX = 1;
			aTiles[nTiles].tStride.nY = a_nSrcStrideY;
			aTiles[nTiles].nPixels = aTiles[nTiles].tSize.nX*aTiles[nTiles].tSize.nY;
			aTiles[nTiles].pData = a_pSrc+(tSrc1.nY-a_tSrcOrigin.nY)*a_nSrcStrideY+tSrc1.nX-a_tSrcOrigin.nX;
			++nTiles;
		}
		if (LONG(a_pOrigin->nX+a_pSize->nX) > tSrc2.nX)
		{
			// right
			aTiles[nTiles].nChannelIDs = EICIRGBA;
			aTiles[nTiles].tOrigin.nX = tSrc2.nX;
			aTiles[nTiles].tOrigin.nY = tSrc1.nY;
			aTiles[nTiles].tSize.nX = a_pOrigin->nX+a_pSize->nX-tSrc2.nX;
			aTiles[nTiles].tSize.nY = tSrc2.nY-tSrc1.nY;
			aTiles[nTiles].tStride.nX = aTiles[nTiles].tStride.nY = 0;
			aTiles[nTiles].nPixels = 1;
			aTiles[nTiles].pData = &a_tDefault;
			++nTiles;
		}
		if (LONG(a_pOrigin->nY+a_pSize->nY) > tSrc2.nY)
		{
			// bottom
			aTiles[nTiles].nChannelIDs = EICIRGBA;
			aTiles[nTiles].tOrigin.nX = a_pOrigin->nX;
			aTiles[nTiles].tOrigin.nY = tSrc2.nY;
			aTiles[nTiles].tSize.nX = a_pSize->nX;
			aTiles[nTiles].tSize.nY = a_pOrigin->nY+a_pSize->nY-tSrc2.nY;
			aTiles[nTiles].tStride.nX = aTiles[nTiles].tStride.nY = 0;
			aTiles[nTiles].nPixels = 1;
			aTiles[nTiles].pData = &a_tDefault;
			++nTiles;
		}
	}
	else
	{
		// out of image
		aTiles[nTiles].nChannelIDs = EICIRGBA;
		aTiles[nTiles].tOrigin = *a_pOrigin;
		aTiles[nTiles].tSize = *a_pSize;
		aTiles[nTiles].tStride.nX = aTiles[nTiles].tStride.nY = 0;
		aTiles[nTiles].nPixels = 1;
		aTiles[nTiles].pData = &a_tDefault;
		++nTiles;
	}
	return a_pVisitor->Visit(nTiles, aTiles, a_pControl);
}

