
#pragma once


class CMemoryStorageFilter :
	public CComObjectRootEx<CComMultiThreadModel>,
	public IStorageFilter,
	public IDataSrcDirect,
	public IDataDstStream
{
public:
#ifdef _DEBUG
	CMemoryStorageFilter() : m_nLocks(0) {}
#endif

	void AppendData(BYTE const* a_pData, size_t a_nLength)
	{
		ATLASSERT(m_nLocks == 0);
		while (a_nLength)
		{
			m_cData.push_back(*a_pData);
			a_pData++;
			a_nLength--;
		}
	}
	ULONG M_Len() const { return m_cData.size(); }
	BYTE const* M_Data() const { return m_cData.size() ? &(m_cData[0]) : NULL; }

	BEGIN_COM_MAP(CMemoryStorageFilter)
		COM_INTERFACE_ENTRY(IStorageFilter)
		COM_INTERFACE_ENTRY(IDataSrcDirect)
		COM_INTERFACE_ENTRY(IDataDstStream)
	END_COM_MAP()

	// IStorageFilter methods
public:
	STDMETHOD(ToText)(IStorageFilter* a_pRoot, BSTR* a_pbstrFilter)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(SubFilterGet)(BSTR a_bstrRelativeLocation, IStorageFilter** a_ppFilter)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(SrcOpen)(IDataSrcDirect** a_ppSrc)
	{
		try
		{
			*a_ppSrc = NULL;
			AddRef();
			*a_ppSrc = this;
			return S_OK;
		}
		catch (...)
		{
			return a_ppSrc == NULL ?  E_POINTER : E_UNEXPECTED;
		}
	}
	STDMETHOD(DstOpen)(IDataDstStream** a_ppDst)
	{
		try
		{
			*a_ppDst = NULL;
			AddRef();
			*a_ppDst = this;
			return S_OK;
		}
		catch (...)
		{
			return a_ppDst == NULL ?  E_POINTER : E_UNEXPECTED;
		}
	}

	// IDataSrcDirect methods
public:
	STDMETHOD(SizeGet)(ULONG* a_pnSize)
	{
		try
		{
			*a_pnSize = static_cast<ULONG>(m_cData.size());
			return S_OK;
		}
		catch (...)
		{
			return a_pnSize == NULL ? E_POINTER : E_UNEXPECTED;
		}
	}
	STDMETHOD(SrcLock)(ULONG a_nOffset, ULONG a_nSize, BYTE const** a_ppBuffer)
	{
		try
		{
			if ((a_nOffset+a_nSize) > m_cData.size())
				return E_FAIL;
			*a_ppBuffer = (&(m_cData[0]))+a_nOffset;
#ifdef _DEBUG
			m_nLocks++;
#endif
			return S_OK;
		}
		catch (...)
		{
			return a_ppBuffer == NULL ? E_POINTER : E_UNEXPECTED;
		}
	}
	STDMETHOD(SrcUnlock)(ULONG UNREF(a_nSize), BYTE const* UNREF(a_pBuffer))
	{
#ifdef _DEBUG
			m_nLocks--;
#endif
			return S_OK;
	}

	// IDataDstStream methods
public:
	STDMETHOD(Write)(ULONG a_nSize, BYTE const* a_pBuffer)
	{
		try
		{
			AppendData(a_pBuffer, a_nSize);
			return S_OK;
		}
		catch (...)
		{
			return E_OUTOFMEMORY;
		}
	}
	STDMETHOD(Seek)(ULONG a_nSize)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(Close)()
	{
		return S_OK;
	}

private:
	std::vector<BYTE> m_cData;

#ifdef _DEBUG
	ULONG m_nLocks;
#endif
};


