
#pragma once

#include <SubjectImpl.h>
#include <ObserverImpl.h>


template<class T, class TBase>
class ATL_NO_VTABLE COutputDeviceImpl :
	public CSubjectImpl<TBase, IOutputDeviceObserver, ULONG>,
	public CObserverImpl<COutputDeviceImpl<T, TBase>, ISegmentObserver, ULONG>,
	public CObserverImpl<COutputDeviceImpl<T, TBase>, IConfigObserver, IUnknown*>
{
public:
	COutputDeviceImpl() :
		m_bBackgroundR(0), m_bBackgroundG(0), m_bBackgroundB(0), m_bBackgroundA(255)
	{
	}
	~COutputDeviceImpl() 
	{
		try
		{
			CLayers::iterator i;
			for (i = m_cLayers.begin(); i != m_cLayers.end(); i++)
			{
				if (i->second.pSegment)
				{
					i->second.pSegment->ObserverDel(CObserverImpl<COutputDeviceImpl<T, TBase>, ISegmentObserver, ULONG>::ObserverGet(), i->first);
				}
				if (i->second.pConfig)
				{
					i->second.pConfig->ObserverDel(CObserverImpl<COutputDeviceImpl<T, TBase>, IConfigObserver, IUnknown*>::ObserverGet(), i->first);
				}
			}
		}
		catch (...)
		{
		}
	}

	// notification forwarding (segment->device)
	void OwnerNotify(TCookie a_tCookie, ULONG)
	{
		Fire_Notify(EODCTLayer0+a_tCookie);
	}
	// notification forwarding (config->device)
	void OwnerNotify(TCookie a_tCookie, IUnknown*)
	{
		Fire_Notify(EODCTConfig);
	}

	// IOutputDevice methods
public:
	STDMETHOD(BackgroundSet)(const TTexelSource* a_ptBackground)
	{
		try
		{
			switch (a_ptBackground->eTypeID)
			{
			case ETSTBytePerChannel:
				m_bBackgroundR = a_ptBackground->b[0];
				m_bBackgroundG = a_ptBackground->b[1];
				m_bBackgroundB = a_ptBackground->b[2];
				m_bBackgroundA = a_ptBackground->b[3];
				break;
			case ETSTFloatPerChannel:
				m_bBackgroundR = static_cast<BYTE>(a_ptBackground->f[0]*255.0f+0.5f)&0xff;
				m_bBackgroundG = static_cast<BYTE>(a_ptBackground->f[1]*255.0f+0.5f)&0xff;
				m_bBackgroundB = static_cast<BYTE>(a_ptBackground->f[2]*255.0f+0.5f)&0xff;
				m_bBackgroundA = static_cast<BYTE>(a_ptBackground->f[3]*255.0f+0.5f)&0xff;
				break;
			default:
				return E_FAIL;
			}

			Fire_Notify(EODCTBackground);

			return S_OK;
		}
		catch (...)
		{
			return E_UNEXPECTED;
		}
	}

	STDMETHOD(CameraSet)(LONG a_nLayer, const TMatrix4x4f* a_ptProj, const TMatrix4x4f* a_ptView)
	{
		try
		{
			if (a_nLayer < 0)
				return E_RW_INVALIDPARAM;

			if (a_ptProj)
			{
				m_cLayers[a_nLayer].tProj = *a_ptProj;
			}

			if (a_ptView)
			{
				m_cLayers[a_nLayer].tView = *a_ptView;
			}

			Fire_Notify(EODCTCamera);
			return S_OK;
		}
		catch (...)
		{
			return E_UNEXPECTED;
		}
	}

	STDMETHOD(CameraGet)(LONG a_nLayer, TMatrix4x4f* a_ptProj, TMatrix4x4f* a_ptView)
	{
		try
		{
			CLayers::const_iterator i = m_cLayers.find(a_nLayer);
			if (i == m_cLayers.end())
				return E_RW_ITEMNOTFOUND;

			if (a_ptProj)
			{
				*a_ptProj = i->second.tProj;
			}

			if (a_ptView)
			{
				*a_ptView = i->second.tView;
			}

			return S_OK;
		}
		catch (...)
		{
			return (a_ptProj == NULL || a_ptView == NULL) ? E_POINTER : E_UNEXPECTED;
		}
	}

	STDMETHOD(SegmentSet)(LONG a_nLayer, ISceneSegment* a_pSegment)
	{
		try
		{
			if (a_nLayer < 0)
				return E_RW_INVALIDPARAM;

			CLayers::iterator i = m_cLayers.find(a_nLayer);
			if (a_pSegment == NULL)
			{
				if (i != m_cLayers.end() && i->second.pSegment != NULL)
				{
					i->second.pSegment->ObserverDel(CObserverImpl<COutputDeviceImpl<T, TBase>, ISegmentObserver, ULONG>::ObserverGet(), a_nLayer);
					i->second.pSegment = NULL;
					Fire_Notify(EODCTLayer0+a_nLayer);
				}
			}
			else
			{
				if (i != m_cLayers.end() && i->second.pSegment != NULL)
				{
					i->second.pSegment->ObserverDel(CObserverImpl<COutputDeviceImpl<T, TBase>, ISegmentObserver, ULONG>::ObserverGet(), a_nLayer);
				}
				m_cLayers[a_nLayer].pSegment = a_pSegment;
				a_pSegment->ObserverIns(CObserverImpl<COutputDeviceImpl<T, TBase>, ISegmentObserver, ULONG>::ObserverGet(), a_nLayer);
				Fire_Notify(EODCTLayer0+a_nLayer);
			}

			return S_OK;
		}
		catch (...)
		{
			return E_UNEXPECTED;
		}
	}

	STDMETHOD(SegmentGet)(LONG a_nLayer, ISceneSegment** a_ppSegment)
	{
		try
		{
			*a_ppSegment = NULL;

			CLayers::const_iterator i = m_cLayers.find(a_nLayer);
			if (i == m_cLayers.end())
				return E_RW_ITEMNOTFOUND;

			if (i->second.pSegment)
			{
				(*a_ppSegment = i->second.pSegment)->AddRef();
			}

			return S_OK;
		}
		catch (...)
		{
			return a_ppSegment == NULL ? E_POINTER : E_UNEXPECTED;
		}
	}

	STDMETHOD(ConfigSet)(LONG a_nLayer, IConfig* a_pConfig)
	{
		try
		{
			if (a_nLayer < 0)
				return E_RW_INVALIDPARAM;

			CLayers::iterator i = m_cLayers.find(a_nLayer);
			if (a_pConfig == NULL)
			{
				if (i != m_cLayers.end() && i->second.pConfig != NULL)
				{
					i->second.pConfig->ObserverDel(CObserverImpl<COutputDeviceImpl<T, TBase>, IConfigObserver, IUnknown*>::ObserverGet(), a_nLayer);
					i->second.pConfig = NULL;
					Fire_Notify(EODCTConfig);
				}
			}
			else
			{
				if (i != m_cLayers.end() && i->second.pConfig != NULL)
				{
					i->second.pConfig->ObserverDel(CObserverImpl<COutputDeviceImpl<T, TBase>, IConfigObserver, IUnknown*>::ObserverGet(), a_nLayer);
				}
				m_cLayers[a_nLayer].pConfig = a_pConfig;
				a_pConfig->ObserverIns(CObserverImpl<COutputDeviceImpl<T, TBase>, IConfigObserver, IUnknown*>::ObserverGet(), a_nLayer);
				Fire_Notify(EODCTConfig);
			}

			return S_OK;
		}
		catch (...)
		{
			return E_UNEXPECTED;
		}
	}

	STDMETHOD(ConfigGet)(LONG a_nLayer, IConfig** a_ppConfig)
	{
		try
		{
			*a_ppConfig = NULL;

			CLayers::const_iterator i = m_cLayers.find(a_nLayer);
			if (i == m_cLayers.end())
				return E_RW_ITEMNOTFOUND;

			if (i->second.pConfig)
			{
				(*a_ppConfig = i->second.pConfig)->AddRef();
			}

			return S_OK;
		}
		catch (...)
		{
			return a_ppConfig == NULL ? E_POINTER : E_UNEXPECTED;
		}
	}

protected:
	struct SLayer
	{
		TMatrix4x4f tProj;
		TMatrix4x4f tView;
		CComPtr<ISceneSegment> pSegment;
		CComPtr<IConfig> pConfig;
	};
	typedef std::map<LONG, SLayer> CLayers;

protected:
	CLayers const& M_Layers() const {return m_cLayers;}
	BYTE M_BackgroundR() const {return m_bBackgroundR;}
	BYTE M_BackgroundG() const {return m_bBackgroundG;}
	BYTE M_BackgroundB() const {return m_bBackgroundB;}
	BYTE M_BackgroundA() const {return m_bBackgroundA;}

private:
	CLayers m_cLayers;
	BYTE m_bBackgroundR;
	BYTE m_bBackgroundG;
	BYTE m_bBackgroundB;
	BYTE m_bBackgroundA;
};

