
#pragma once

#include <SceneAssistantImpl.h>


extern __declspec(selectany) LONG const CFGVAL_SCENEASSISTCULL_FOV = 1;

template<class TSceneAssistant, class TOcclusionBuilder>
class CFOVCulling :
	public TSceneAssistant,
	public TOcclusionBuilder
{
public:
	void Init(float UNREF(a_fLODFactor), int UNREF(a_nWndSize), TMatrix4x4f const& a_tProjection)
	{
		TMatrix4x4f tInv;
		Matrix4x4fInverse(a_tProjection, &tInv);
		for (int i = 0; i < 8; i++)
		{
			TPoint t = {((i&1)<<1)-1, (i&2)-1, ((i&4)>>1)-1};
			TransformPoint(tInv, &t, m_aPoints+i);
		}
	}
	enum { CFGVAL = CFGVAL_SCENEASSISTCULL_FOV };
	static LPWSTR Name() { return L"[0409]Field of view culling[0405]Oezn podle zbru kamery"; }


	// ISceneAssistant methods
public:
	STDMETHOD(ObjectTestTriangles)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData))
	{
		return S_OK;
	}
	STDMETHOD(ObjectTestPyramids)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData))
	{
		return S_OK;
	}
	STDMETHOD(ObjectTestSpheres)(ULONG a_nSpheres, TMatrix4x4f const* a_pPositions)
	{
		try
		{
			for (ULONG i = 0; i < a_nSpheres; i++)
			{
				TMatrix4x4f tInv;
				Matrix4x4fInverse(a_pPositions[i], &tInv);
				TPoint tBounds[8];
				for (int j = 0; j < 8; j++)
				{
					TransformPoint(tInv, m_aPoints+j, tBounds+j);
				}
				struct SPlane {int i1, i2, i3;} aPlanes[6] =
				{
					{0, 1, 2}, // front
					{5, 4, 7}, // back
					{1, 5, 3}, // right
					{4, 0, 2}, // left
					{2, 3, 6}, // top
					{4, 5, 0}, // bottom
				};
				int k;
				for (k = 0; k < 6; k++)
				{
					TPoint p1 = tBounds[aPlanes[k].i1];
					TPoint p2 = tBounds[aPlanes[k].i2];
					TPoint p3 = tBounds[aPlanes[k].i3];
					p2.x -= p1.x; p2.y -= p1.y; p2.z -= p1.z;
					p3.x -= p1.x; p3.y -= p1.y; p3.z -= p1.z;
					TPoint tNormal =
					{
						p2.y*p3.z - p3.y*p2.z,
						p2.z*p3.x - p3.z*p2.x,
						p2.x*p3.y - p3.x*p2.y
					};
					float fMag = 1.0f/sqrtf(tNormal.x*tNormal.x + tNormal.y*tNormal.y + tNormal.z*tNormal.z);
					tNormal.x *= fMag; tNormal.y *= fMag; tNormal.z *= fMag;
					// projection into direction
					p1.x += tNormal.x; p1.y += tNormal.y; p1.z += tNormal.z;
					if ((p1.x*tNormal.x + p1.y*tNormal.y + p1.z*tNormal.z) < 0)
						break;
				}
				if (k == 6)
					return S_OK;
			}

			return S_FALSE;
		}
		catch (...)
		{
			return a_pPositions == NULL ? E_POINTER : E_UNEXPECTED;
		}
	}
	STDMETHOD(CameraTestPyramids)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData))
	{
		return S_OK;
	}
	STDMETHOD(CameraTestSpheres)(ULONG UNREF(a_nSpheres), TMatrix4x4f const* UNREF(a_tPosition))
	{
		return S_OK;
	}

	// IOcclusionBuilder methods
public:
	STDMETHOD(AddTriangles)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData), ULONG UNREF(a_nIndices), ULONG const* UNREF(a_pIndices))
	{
		return E_NOTIMPL;
	}
	STDMETHOD(AddPyramids)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData), ULONG UNREF(a_nIndices), ULONG const* UNREF(a_pIndices))
	{
		return E_NOTIMPL;
	}

private:
	struct TPoint {float x,y,z;};

	void TransformPoint(TMatrix4x4f const& t, TPoint const* a_pSrc, TPoint* a_pDst) const
	{
		float w = 1.0f/(t._14*a_pSrc->x + t._24*a_pSrc->y + t._34*a_pSrc->z + t._44);
		a_pDst->x = (t._11*a_pSrc->x + t._21*a_pSrc->y + t._31*a_pSrc->z + t._41)*w;
		a_pDst->y = (t._12*a_pSrc->x + t._22*a_pSrc->y + t._32*a_pSrc->z + t._42)*w;
		a_pDst->z = (t._13*a_pSrc->x + t._23*a_pSrc->y + t._33*a_pSrc->z + t._43)*w;
	}

private:
	TPoint m_aPoints[8];
};

