
#pragma once

extern __declspec(selectany) OLECHAR const CFGID_SCENEASSISTLOD[] = L"AssistLOD";
extern __declspec(selectany) OLECHAR const CFGID_SCENEASSISTCULL[] = L"AssistCull";
extern __declspec(selectany) OLECHAR const CFGID_LODFACTOR[] = L"LODFactor";


template<template<class> class TLOD, template<class, class> class TCull>
class CSceneAssistant : 
	public CComObjectRootEx<CComMultiThreadModel>,
	public TCull<TLOD<ISceneAssistant>, IOcclusionBuilder >
{
public:
	CSceneAssistant()
	{
	}

	void Init(float a_fLODFactor, int a_nWndSize, TMatrix4x4f const& a_tProjection)
	{
		TCull<TLOD<ISceneAssistant>, IOcclusionBuilder>::Init(a_fLODFactor, a_nWndSize, a_tProjection);
		TLOD<ISceneAssistant>::Init(a_fLODFactor, a_nWndSize, a_tProjection);
	}

BEGIN_COM_MAP(CSceneAssistant)
	COM_INTERFACE_ENTRY(ISceneAssistant)
	COM_INTERFACE_ENTRY(IOcclusionBuilder)
END_COM_MAP()

};


extern __declspec(selectany) LONG const CFGVAL_SCENEASSISTCULL_NONE = 0;

template<class TSceneAssistant, class TOcclusionBuilder>
class CNoCulling :
	public TSceneAssistant,
	public TOcclusionBuilder
{
public:
	void Init(float UNREF(a_fLODFactor), int UNREF(a_nWndSize), TMatrix4x4f const& UNREF(a_tProjection))
	{
	}
	enum { CFGVAL = CFGVAL_SCENEASSISTCULL_NONE };
	static LPWSTR Name() { return L"[0409]No culling[0405]dn oezvn"; }

	// ISceneAssistant methods
public:
	STDMETHOD(ObjectTestTriangles)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData))
	{
		return S_OK;
	}
	STDMETHOD(ObjectTestPyramids)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData))
	{
		return S_OK;
	}
	STDMETHOD(ObjectTestSpheres)(ULONG UNREF(a_nSpheres), TMatrix4x4f const* UNREF(a_tPosition))
	{
		return S_OK;
	}
	STDMETHOD(CameraTestPyramids)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData))
	{
		return S_OK;
	}
	STDMETHOD(CameraTestSpheres)(ULONG UNREF(a_nSpheres), TMatrix4x4f const* UNREF(a_tPosition))
	{
		return S_OK;
	}

	// IOcclusionBuilder methods
public:
	STDMETHOD(AddTriangles)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData), ULONG UNREF(a_nIndices), ULONG const* UNREF(a_pIndices))
	{
		return E_NOTIMPL;
	}
	STDMETHOD(AddPyramids)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData), ULONG UNREF(a_nIndices), ULONG const* UNREF(a_pIndices))
	{
		return E_NOTIMPL;
	}
};


extern __declspec(selectany) LONG const CFGVAL_SCENEASSISTLOD_CONSTANT = 0;

template<class TSceneAssistant>
class CConstantLOD :
	public TSceneAssistant
{
public:
	void Init(float a_fLODFactor, int UNREF(a_nWndSize), TMatrix4x4f const& UNREF(a_tProjection))
	{
		m_fLOD = a_fLODFactor*150.0f; // assume everywhere in the scene 1 world length unit is 150 pixels
	}
	enum { CFGVAL = CFGVAL_SCENEASSISTLOD_CONSTANT };
	static LPWSTR Name() { return L"[0409]Constant LOD[0405]Konstantn LOD"; }

	// ISceneAssistant methods
public:
	STDMETHOD(LODAt)(TMatrix4x4f const* UNREF(a_tPosition), float* a_pfLOD)
	{
		try
		{
			*a_pfLOD = m_fLOD;
			return S_OK;
		}
		catch (...)
		{
			return a_pfLOD == NULL ? E_POINTER : E_UNEXPECTED;
		}
	}

private:
	float m_fLOD;
};




template<template<class> class TThis, class TNext>
class CLODTypeList
{
public:
	template<template<class, class> class TCull>
	static void CreateAssistant(LONG a_nLOD, float a_fLODFactor, int a_nWndSize, TMatrix4x4f const& a_tProjection, CComPtr<ISceneAssistant>& a_pAssist, CComPtr<IOcclusionBuilder>& a_pBuilder)
	{
		if (TThis<ISceneAssistant>::CFGVAL == a_nLOD) // the template arguments are not correct here, but we only need to access the enum, so it does not really matter
		{
			CComObject<CSceneAssistant<TThis, TCull> >* p = NULL;
			CComObject<CSceneAssistant<TThis, TCull> >::CreateInstance(&p);
			a_pAssist = p;
			a_pBuilder = p;
			p->Init(a_fLODFactor, a_nWndSize, a_tProjection);
		}
		else
		{
			TNext::CreateAssistant<TCull>(a_nLOD, a_fLODFactor, a_nWndSize, a_tProjection, a_pAssist, a_pBuilder);
		}
	}
	static void InsertConfigOptions(IConfigWithDependencies* a_pInitCfg, BSTR a_bstrCfgID)
	{
		HRESULT hRes = a_pInitCfg->ItemOptionAdd(a_bstrCfgID, CConfigValue(static_cast<LONG>(TThis<ISceneAssistant>::CFGVAL)), CMultiLanguageString::GetAuto(TThis<ISceneAssistant>::Name()), 0, NULL);
		if (FAILED(hRes)) throw hRes;
		TNext::InsertConfigOptions(a_pInitCfg, a_bstrCfgID);
	}
};

class CLODTypeListEnd
{
public:
	template<template<class, class> class TCull>
	static void CreateAssistant(LONG, float, int, TMatrix4x4f const&, CComPtr<ISceneAssistant>&, CComPtr<IOcclusionBuilder>&)
	{
		throw E_FAIL;
	}
	static void InsertConfigOptions(IConfigWithDependencies*, BSTR)
	{
	}
};


template<template<class, class> class TThis, class TNext>
class CCullTypeList
{
public:
	template<class TLODTypeList>
	static void CreateAssistant(LONG a_nLOD, LONG a_nCull, float a_fLODFactor, int a_nWndSize, TMatrix4x4f const& a_tProjection, CComPtr<ISceneAssistant>& a_pAssist, CComPtr<IOcclusionBuilder>& a_pBuilder)
	{
		if (TThis<ISceneAssistant, IOcclusionBuilder>::CFGVAL == a_nCull) // the template arguments are not correct here, but we only need to access the enum, so it does not really matter
		{
			TLODTypeList::CreateAssistant<TThis>(a_nLOD, a_fLODFactor, a_nWndSize, a_tProjection, a_pAssist, a_pBuilder);
		}
		else
		{
			TNext::CreateAssistant<TLODTypeList>(a_nLOD, a_nCull, a_fLODFactor, a_nWndSize, a_tProjection, a_pAssist, a_pBuilder);
		}
	}
	static void InsertConfigOptions(IConfigWithDependencies* a_pInitCfg, BSTR a_bstrCfgID)
	{
		HRESULT hRes = a_pInitCfg->ItemOptionAdd(a_bstrCfgID, CConfigValue(static_cast<LONG>(TThis<ISceneAssistant, IOcclusionBuilder>::CFGVAL)), CMultiLanguageString::GetAuto(TThis<ISceneAssistant, IOcclusionBuilder>::Name()), 0, NULL);
		if (FAILED(hRes)) throw hRes;
		TNext::InsertConfigOptions(a_pInitCfg, a_bstrCfgID);
	}
};

class CCullTypeListEnd
{
public:
	template<class TLODTypeList>
	static void CreateAssistant(LONG, LONG, float, int, TMatrix4x4f const&, CComPtr<ISceneAssistant>&, CComPtr<IOcclusionBuilder>&)
	{
		throw E_FAIL;
	}
	static void InsertConfigOptions(IConfigWithDependencies*, BSTR)
	{
	}
};

template<class TCullTypeList, class TLODTypeList>
class CSceneAssistantUtils
{
public:
	static void CreateAssistant(LONG a_nLOD, LONG a_nCull, float a_fLODFactor, int a_nWndSize, TMatrix4x4f const& a_tProjection, CComPtr<ISceneAssistant>& a_pAssist, CComPtr<IOcclusionBuilder>& a_pBuilder)
	{
		TCullTypeList::CreateAssistant<TLODTypeList>(a_nLOD, a_nCull, a_fLODFactor, a_nWndSize, a_tProjection, a_pAssist, a_pBuilder);
	}
	static void CreateAssistant(IConfig* a_pConfig, float a_fLODFactor, int a_nWndSize, TMatrix4x4f const& a_tProjection, CComPtr<ISceneAssistant>& a_pAssist, CComPtr<IOcclusionBuilder>& a_pBuilder)
	{
		CreateAssistant(GetLODAssistantID(a_pConfig), GetCullAssistantID(a_pConfig), a_fLODFactor, a_nWndSize, a_tProjection, a_pAssist, a_pBuilder);
	}
	static void CreateAssistant(IConfig* a_pConfig, IConfig* a_pConfigLODFactor, int a_nWndSize, TMatrix4x4f const& a_tProjection, CComPtr<ISceneAssistant>& a_pAssist, CComPtr<IOcclusionBuilder>& a_pBuilder)
	{
		CreateAssistant(GetLODAssistantID(a_pConfig), GetCullAssistantID(a_pConfig), GetLODFactor(a_pConfigLODFactor), a_nWndSize, a_tProjection, a_pAssist, a_pBuilder);
	}
	static void CreateAssistant(LONG a_nLOD, LONG a_nCull, IConfig* a_pConfigLODFactor, int a_nWndSize, TMatrix4x4f const& a_tProjection, CComPtr<ISceneAssistant>& a_pAssist, CComPtr<IOcclusionBuilder>& a_pBuilder)
	{
		CreateAssistant(a_nLOD, a_nCull, GetLODFactor(a_pConfigLODFactor), a_nWndSize, a_tProjection, a_pAssist, a_pBuilder);
	}

	static LONG GetLODAssistantID(IConfig* a_pConfig)
	{
		CConfigValue cLOD;
		a_pConfig->ItemValueGet(CComBSTR(CFGID_SCENEASSISTLOD), &cLOD);
		return cLOD;
	}
	static LONG GetCullAssistantID(IConfig* a_pConfig)
	{
		CConfigValue cCull;
		a_pConfig->ItemValueGet(CComBSTR(CFGID_SCENEASSISTCULL), &cCull);
		return cCull;
	}
	static float GetLODFactor(IConfig* a_pConfig)
	{
		CConfigValue cLODFactor;
		a_pConfig->ItemValueGet(CComBSTR(CFGID_LODFACTOR), &cLODFactor);
		return cLODFactor;
	}

	static void InitLODFactorConfig(IConfigWithDependencies* a_pInitCfg, float a_fDefault = 1.0f)
	{
		HRESULT hRes = a_pInitCfg->ItemInsSimple(CComBSTR(CFGID_LODFACTOR), CMultiLanguageString::GetAuto(L"[0409]LOD Factor[0405]LOD faktor"), CMultiLanguageString::GetAuto(L"[0409]The actual LOD value at each point in space is multiplied by this value.[0405]Hodnota LOD v kadm bodu prostoru bude vynsobena zadanou hodnotou."), CConfigValue(a_fDefault), NULL, 0, NULL);
		if (FAILED(hRes)) throw hRes;
	}
	static void InitLODAndCullConfig(IConfigWithDependencies* a_pInitCfg, LONG a_nDefaultLOD = CFGVAL_SCENEASSISTLOD_CONSTANT, LONG a_nDefaultCull = CFGVAL_SCENEASSISTCULL_NONE)
	{
		CComBSTR cCFGID_SCENEASSISTLOD(CFGID_SCENEASSISTLOD);
		HRESULT hRes = a_pInitCfg->ItemIns1ofN(cCFGID_SCENEASSISTLOD, CMultiLanguageString::GetAuto(L"[0409]LOD Algorithm[0405]Vpoet LOD"), CMultiLanguageString::GetAuto(L"[0409]The method for LOD (Level of Detail) computation. Each point in space is assigned a detail level value determined by the selected method.[0405]Zpsob vpotu hodnoty LOD (rove detail). Kadmu bodu prostor je piazena hodnota rovn detail podle zvolen metody."), CConfigValue(a_nDefaultLOD), NULL);
		if (FAILED(hRes)) throw hRes;
		TLODTypeList::InsertConfigOptions(a_pInitCfg, cCFGID_SCENEASSISTLOD);

		CComBSTR cCFGID_SCENEASSISTCULL(CFGID_SCENEASSISTCULL);
		hRes = a_pInitCfg->ItemIns1ofN(cCFGID_SCENEASSISTCULL, CMultiLanguageString::GetAuto(L"[0409]Culling Method[0405]Metoda oezvn"), CMultiLanguageString::GetAuto(L"[0409]The method for determining wheter a specified object is visible and should be rendered or not.[0405]Zpsob zjiovn, je-li dan objekt viditeln nebo m-li bt oznut."), CConfigValue(a_nDefaultLOD), NULL);
		if (FAILED(hRes)) throw hRes;
		TCullTypeList::InsertConfigOptions(a_pInitCfg, cCFGID_SCENEASSISTCULL);
	}
};

