
#pragma once

#include <SceneAssistantImpl.h>
#include <math.h>


extern __declspec(selectany) LONG const CFGVAL_SCENEASSISTLOD_PIXELSIZE = 1;

template<class TSceneAssistant>
class CPixelSizeLOD :
	public TSceneAssistant
{
public:
	void Init(float a_fLODFactor, int a_nWndSize, TMatrix4x4f const& a_tProjection)
	{
		m_fLODFactor = a_fLODFactor;
		m_tProjection = a_tProjection;

		if (a_nWndSize == 0)
			a_nWndSize = 1;

		static float const s0N0[] = {0.0f, -1.0f, 0.0f};
		static float const s0P0[] = {0.0f,  1.0f, 0.0f};
		static float const s0N1[] = {0.0f, -1.0f, 1.0f};
		static float const s0P1[] = {0.0f,  1.0f, 1.0f};
		float w0N0[3];
		float w0P0[3];
		float w0N1[3];
		float w0P1[3];
		TransformVector(s0N0, w0N0);
		TransformVector(s0P0, w0P0);
		TransformVector(s0N1, w0N1);
		TransformVector(s0P1, w0P1);
		m_fNear = w0N0[2];
		m_fDist = 1.0f/(w0N1[2]-m_fNear);
		m_fSizeNear = fabsf(w0P0[1]-w0N0[1])/a_nWndSize;
		m_fSizeDelta = fabsf(w0P1[1]-w0N1[1])/a_nWndSize-m_fSizeNear;
	}
	enum { CFGVAL = CFGVAL_SCENEASSISTLOD_PIXELSIZE };
	static LPWSTR Name() { return L"[0409]Object pixel size LOD[0405]LOD podle velikosti objektu v pixelech"; }

	// ISceneAssistant methods
public:
	STDMETHOD(LODAt)(TMatrix4x4f const* a_tPosition, float* a_pfLOD)
	{
		try
		{
			float fZoom = sqrtf((a_tPosition->_11*a_tPosition->_11)+(a_tPosition->_21*a_tPosition->_21)+(a_tPosition->_31*a_tPosition->_31)) +
						sqrtf((a_tPosition->_12*a_tPosition->_12)+(a_tPosition->_22*a_tPosition->_22)+(a_tPosition->_32*a_tPosition->_32)) +
						sqrtf((a_tPosition->_13*a_tPosition->_13)+(a_tPosition->_23*a_tPosition->_23)+(a_tPosition->_33*a_tPosition->_33));

			float fZ = (a_tPosition->_43-m_fNear)*m_fDist;
			if (fZ < 0.0f) fZ = 0.0f;
			if (fZ > 1.0f) fZ = 1.0f;
			float fDepthFactor = 1.0f/(fZ*m_fSizeDelta + m_fSizeNear);

			*a_pfLOD = m_fLODFactor * fDepthFactor * 0.333333f * fZoom;

			return S_OK;
		}
		catch (...)
		{
			return a_tPosition == NULL || a_pfLOD == NULL ? E_POINTER : E_UNEXPECTED;
		}
	}

private:
	void TransformVector(float const* a_pSrc, float* a_pDst) const
	{
		TMatrix4x4f t;
		Matrix4x4fInverse(m_tProjection, &t);
		float w = 1.0f/(t._14*a_pSrc[0] + t._24*a_pSrc[1] + t._34*a_pSrc[2] + t._44);
		a_pDst[0] = (t._11*a_pSrc[0] + t._21*a_pSrc[1] + t._31*a_pSrc[2] + t._41)*w;
		a_pDst[1] = (t._12*a_pSrc[0] + t._22*a_pSrc[1] + t._32*a_pSrc[2] + t._42)*w;
		a_pDst[2] = (t._13*a_pSrc[0] + t._23*a_pSrc[1] + t._33*a_pSrc[2] + t._43)*w;
	}

private:
	float m_fLODFactor;
	TMatrix4x4f m_tProjection;
	float m_fNear;
	float m_fDist;
	float m_fSizeNear;
	float m_fSizeDelta;
};

