
#pragma once

class ATL_NO_VTABLE CSegmentBoundingBox :
	public CComObjectRootEx<CComMultiThreadModel>,
	public ISceneAssistant,
	public ISceneBuilder
{
public:
	CSegmentBoundingBox()
	{
		Reset();
	}

	BEGIN_COM_MAP(CSegmentBoundingBox)
		COM_INTERFACE_ENTRY(ISceneAssistant)
		COM_INTERFACE_ENTRY(ISceneBuilder)
	END_COM_MAP()

	void Reset()
	{
		m_fX1 = 1e30f;
		m_fX2 = -1e30f;
		m_fY1 = 1e30f;
		m_fY2 = -1e30f;
		m_fZ1 = 1e30f;
		m_fZ2 = -1e30f;
		m_tTrans = TMATRIX4X4F_IDENTITY;
	}
	void ProcessSegment(ISceneSegment* a_pSegment, TMatrix4x4f const& a_tView)
	{
		a_pSegment->ProcessGeometry(this, this, &a_tView, NULL);
	}
	float M_XMin() const { return m_fX1; }
	float M_YMin() const { return m_fY1; }
	float M_ZMin() const { return m_fZ1; }
	float M_XMax() const { return m_fX2; }
	float M_YMax() const { return m_fY2; }
	float M_ZMax() const { return m_fZ2; }

	// ISceneAssistant methods
public:
	STDMETHOD(LODAt)(TMatrix4x4f const* UNREF(a_tPosition), float* a_pfLOD)
	{
		*a_pfLOD = 1.0f;
		return S_OK;
	}
	STDMETHOD(ObjectTestTriangles)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData))
	{
		return S_OK;
	}
	STDMETHOD(ObjectTestPyramids)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData))
	{
		return S_OK;
	}
	STDMETHOD(ObjectTestSpheres)(ULONG UNREF(a_nSpheres), TMatrix4x4f const* UNREF(a_tPosition))
	{
		return S_OK;
	}
	STDMETHOD(CameraTestPyramids)(ULONG UNREF(a_nVertices), ULONG UNREF(a_nStride), BYTE const* UNREF(a_pData))
	{
		return S_OK;
	}
	STDMETHOD(CameraTestSpheres)(ULONG UNREF(a_nSpheres), TMatrix4x4f const* UNREF(a_tPosition))
	{
		return S_OK;
	}

	// ISceneBuilder methods
public:
	STDMETHOD(PixelProcLoad)(BSTR UNREF(a_bstrUID), ULONG UNREF(a_nTimeStamp))
	{
		return S_OK;
	}
	STDMETHOD(PixelProcReset)()
	{
		return S_OK;
	}
	STDMETHOD(PixelProcSet)(BSTR UNREF(a_bstrUID), ULONG UNREF(a_nTimeStamp), ULONG UNREF(a_nTexelSources), const TTexelSource* UNREF(a_aTexelSources))
	{
		return S_OK;
	}

	STDMETHOD(VertexProcLoad)(BSTR a_bstrUID, ULONG a_nTimeStamp)
	{
		return S_OK;
	}
	STDMETHOD(VertexProcSetSimple)(BSTR a_bstrUID, ULONG a_nTimeStamp, ULONG a_nVertexFormatFlags)
	{
		return S_OK;
	}
	STDMETHOD(VertexProcSetProgram)(BSTR a_bstrUID, ULONG a_nTimeStamp, IUnknown* a_pProgram)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(VertexConstantsSet)(ULONG a_nConstants, BYTE const* a_pConstants)
	{
		if (a_nConstants >= sizeof m_tTrans)
			CopyMemory(&m_tTrans, a_pConstants, sizeof m_tTrans);
		return S_OK;
	}

	STDMETHOD(GeometryLoad)(BSTR UNREF(a_bstrUID), ULONG UNREF(a_nTimeStamp))
	{
		return E_NOTIMPL;
	}

	STDMETHOD(Triangles)(BSTR a_bstrUID, ULONG a_nTimeStamp, ULONG a_nVertices, ULONG a_nStride, BYTE const* a_pData, ULONG a_nIndices, ULONG const* a_pIndices)
	{
		for (ULONG i = 0; i < a_nVertices; ++i, a_pData += a_nStride)
		{
			TVector3f t = TransformVector3(m_tTrans, *reinterpret_cast<TVector3f const*>(a_pData));
			if (m_fX1 > t.x) m_fX1 = t.x;
			if (m_fY1 > t.y) m_fY1 = t.y;
			if (m_fZ1 > t.z) m_fZ1 = t.z;
			if (m_fX2 < t.x) m_fX2 = t.x;
			if (m_fY2 < t.y) m_fY2 = t.y;
			if (m_fZ2 < t.z) m_fZ2 = t.z;
		}
		return S_OK;
	}
	STDMETHOD(NTriangles)(BSTR a_bstrUID, ULONG a_nTimeStamp, ULONG a_nVertices, ULONG a_nStride, BYTE const* a_pData, ULONG a_nIndices, ULONG const* a_pIndices)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(BezierTriangles)(BSTR a_bstrUID, ULONG a_nTimeStamp, ULONG a_nDegree, ULONG nVertices, ULONG nStride, BYTE const* pData, ULONG a_nIndices, ULONG const* a_pIndices)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(BezierQuads)(BSTR a_bstrUID, ULONG a_nTimeStamp, ULONG a_nDegreeU, ULONG a_nDegreeV, ULONG nVertices, ULONG nStride, BYTE const* pData, ULONG a_nIndices, ULONG const* a_pIndices)
	{
		return E_NOTIMPL;
	}

	STDMETHOD(Pyramids)(BSTR a_bstrUID, ULONG a_nTimeStamp, ULONG a_nVertices, ULONG a_nStride, BYTE const* a_pData, ULONG a_nIndices, ULONG const* a_pIndices)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(BezierPyramids)(BSTR a_bstrUID, ULONG a_nTimeStamp, ULONG a_nVertices, ULONG a_nStride, BYTE const* a_pData, ULONG a_nIndices, ULONG const* a_pIndices)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(Spheres)(BSTR a_bstrUID, ULONG a_nTimeStamp, ULONG a_nVertices, ULONG a_nStride, BYTE const* a_pData, ULONG a_nIndices, ULONG const* a_pIndices)
	{
		return E_NOTIMPL;
	}

private:
	TMatrix4x4f m_tTrans;
	float m_fX1;
	float m_fX2;
	float m_fY1;
	float m_fY2;
	float m_fZ1;
	float m_fZ2;
};

